@@ -53,43 +53,8 @@ iblkcol_lag <- function(M, ...) {
 
 
 
-## Internal function: check data and scale 
+# Internal function: Sure Independent Screening for hima
 
-dat <- data.frame(y = 1:12, x = c(rep("A",3), rep("B", 3), rep("C", 3), rep("D", 3)))
-dat
-f <- y~x
-
-formula <- f
-model.matrix(f, dat)
-
-model.matrix(update(f, .~.-1), dat)
-
-
-checkscale <- function(formula, dat) {
-  if (is.null(dat)) 
-    return(list(dn = NULL, d = NULL, ds = NULL))
-  if(sum(is.na(dat))>0)
-    return("Missing")
-  
-  
-  
-  dat_scale <- scale(dat)
-  dat_names <- names(dat)
-  if (any(class(dat) %in% c("matrix", "data.frame", "data.table"))) {
-    dat_names <- colnames(dat)
-    dat <- as.matrix(data.frame(dat_scale))
-  } else {
-    dat_names <- names(dat)
-    dat <- as.numeric(dat_scale)
-  }
-  dat_scale <- as.numeric(attributes(dat_scale)[["scaled:scale"]])
-  return(list(dn = dat_names, d = dat, ds = dat_scale))
-}
-
-
-
-# Internal function: Sure Independent Screening
-# Global variables:
 globalVariables("n")
 globalVariables("M_chunk")
 
@@ -233,7 +198,7 @@ LDPE_func <- function(ID, X, OT, status){
 
 
 
-# Internal function: LDPE
+# Internal function: null_estimation
 # A function to estimate the proportions of the three component nulls
 # This is from HDMT package version < 1.0.4
 
@@ -360,3 +325,110 @@ rdirichlet <- function (n = 1, alpha)
   for (i in 1:length(alpha)) Gam[, i] <- stats::rgamma(n, shape = alpha[i])
   Gam/rowSums(Gam)
 }
+
+
+
+# Internal function: DACT
+# A function to perform Divide-Aggregate Composite-null Test (DACT) by Liu et al. (2020).
+# p value is corrected by JC method Jin and Cai (2007).
+# This function is used in eHIMA
+
+DACT = function(p_a,p_b){
+  Z_a = stats::qnorm(p_a,lower.tail = F)
+  Z_b = stats::qnorm(p_b,lower.tail = F)
+  pi0a = 1 - .nonnullPropEst(Z_a,0,1)
+  pi0b = 1 - .nonnullPropEst(Z_b,0,1)
+  if(pi0a > 1){
+    pi0a = 1
+  }
+  if(pi0b >1){
+    pi0b = 1
+  }
+  p.mat = cbind(p_a,p_b)
+  p3 = (apply(p.mat,1,max))^2
+  wg1 = pi0a*(1-pi0b)
+  wg2 = (1-pi0a)*pi0b
+  wg3 = pi0a*pi0b
+  wg.sum = wg1 + wg2 + wg3
+  wg.std = c(wg1,wg2,wg3)/wg.sum
+  p_dact = wg.std[1]*p_a + wg.std[2]*p_b + wg.std[3]*p3
+  p_dact = .JCCorrect(p_dact)
+  return(p_dact)
+}
+
+.JCCorrect = function(pval){
+  z = stats::qnorm(pval,lower.tail = F)
+  res= .nullParaEst(z)
+  pval.JC = stats::pnorm(z,mean = res$mu,sd = res$s,lower.tail = F)
+  return(pval.JC)
+}
+
+.nonnullPropEst <- function(x,u,sigma)
+{
+  z  = (x - u)/sigma
+  xi = c(0:100)/100
+  tmax=sqrt(log(length(x)))
+  tt=seq(0,tmax,0.1)
+  
+  epsest=NULL
+  
+  for (j in 1:length(tt)) {
+    
+    t=tt[j]
+    f  = t*xi
+    f  = exp(f^2/2)
+    w  = (1 - abs(xi))
+    co  = 0*xi
+    
+    for (i in 1:101) {
+      co[i] = mean(cos(t*xi[i]*z));
+    }
+    epshat = 1 - sum(w*f*co)/sum(w)
+    epsest=c(epsest,epshat)
+  }
+  return(epsest=max(epsest))
+}
+
+.nullParaEst<-function (x,gamma=0.1)
+{
+  n = length(x)
+  t = c(1:1000)/200
+  
+  gan    = n^(-gamma)
+  that   = 0
+  shat   = 0
+  uhat   = 0
+  epshat = 0
+  
+  phiplus   = rep(1,1000)
+  phiminus  = rep(1,1000)
+  dphiplus  = rep(1,1000)
+  dphiminus = rep(1,1000)
+  phi       = rep(1,1000)
+  dphi      = rep(1,1000)
+  
+  for (i in 1:1000) {
+    s = t[i]
+    phiplus[i]   = mean(cos(s*x))
+    phiminus[i]  = mean(sin(s*x))
+    dphiplus[i]  = -mean(x*sin(s*x))
+    dphiminus[i] = mean(x*cos(s*x))
+    phi[i]       = sqrt(phiplus[i]^2 + phiminus[i]^2)
+  }
+  
+  ind = min(c(1:1000)[(phi - gan) <= 0])
+  tt = t[ind]
+  a  = phiplus[ind]
+  b  = phiminus[ind]
+  da = dphiplus[ind]
+  db = dphiminus[ind]
+  c  = phi[ind]
+  
+  that   = tt
+  shat   = -(a*da + b*db)/(tt*c*c)
+  shat   = sqrt(shat)
+  uhat   = -(da*b - db*a)/(c*c)
+  epshat = 1 - c*exp((tt*shat)^2/2)
+  
+  return(musigma=list(mu=uhat,s=shat))
+}
